/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2011-2012 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.jonas.admin.webservicesconsole.service;

import com.eviware.soapui.impl.WsdlInterfaceFactory;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.WsdlSubmit;
import com.eviware.soapui.impl.wsdl.WsdlSubmitContext;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.support.SoapUIException;
import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Instantiate;
import org.apache.felix.ipojo.annotations.Invalidate;
import org.apache.felix.ipojo.annotations.Provides;
import org.apache.felix.ipojo.annotations.ServiceProperty;
import org.apache.felix.ipojo.annotations.Validate;
import org.apache.xmlbeans.XmlException;
import org.ow2.kerneos.core.service.KerneosSimpleService;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Web Services console service.
 */
@Component
@Instantiate
@Provides
public class WebServicesConsoleService implements KerneosSimpleService {
    @ServiceProperty(name = KerneosSimpleService.ID, value = "webServicesConsole")
    private String id;

    /**
     * Logger of the class
     */
    private Log logger = LogFactory.getLog(WebServicesConsoleService.class);

    public WebServicesConsoleService() {
    }

    @Validate
    private void start() {
        logger.debug("");
    }

    @Invalidate
    private void stop() {
        logger.debug("");
    }

    /**
     * Gets all the available operations and return the list.
     *
     * @param p_Url the web service URL
     * @return the list of available operations
     * @throws Exception if something happens
     */
    public List<String> getOperations(String p_Url) throws Exception {
        this.logger.info("Getting SOAP operation list for webservice at {0} ", p_Url);
        List<String> result = new ArrayList<String>();

        try {
            WsdlProject project = new WsdlProject();
            WsdlInterface ife = WsdlInterfaceFactory.importWsdl(project, p_Url, true)[0];

            List<Operation> operation = ife.getOperationList();

            for (Operation op : operation) {
                result.add(op.getName());
            }

        } catch (Exception e) {
            this.logger.error("An error occurred while getting operations' list for the URL {0}.", p_Url);
            this.logger.error(e);
            throw e;
        }
        return result;
    }

    /**
     * Get a soap request skeleton to be filled in by the user.
     *
     * @param p_Url       the web service URL
     * @param p_Operation the web service operation
     * @return a soap request skeleton
     */
    public String getRequest(String p_Url, String p_Operation) {
        this.logger.info("Getting SOAP request for webservice at {0} for operation {1}.", p_Url, p_Operation);

        String result = "Error";
        try {
            WsdlProject project = new WsdlProject();
            WsdlInterface ife = WsdlInterfaceFactory.importWsdl(project, p_Url, true)[0];

            List<Operation> operation = ife.getOperationList();

            WsdlOperation sayHelloOperation = ife.getOperationByName(p_Operation);
            result = sayHelloOperation.createRequest(true);

        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (SoapUIException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }


        return result;
    }

    /**
     * Send the given soap request to the webservice and waits for a response.
     *
     * @param p_Url       the web service URL
     * @param p_Operation the web service operation
     * @param p_Request   the soap request to send
     * @return the response from the webservice.
     */
    public String sendRequest(String p_Url, String p_Operation, String p_Request) {
        this.logger.info("Sending SOAP request for webservice at {0} for operation {1}.", p_Url, p_Operation);

        String result = "Error";
        try {
            WsdlProject project = new WsdlProject();
            WsdlInterface ife = WsdlInterfaceFactory.importWsdl(project, p_Url, true)[0];

            List<Operation> operation = ife.getOperationList();

            WsdlOperation sayHelloOperation = ife.getOperationByName(p_Operation);
            WsdlRequest request = sayHelloOperation.addNewRequest("My request" + p_Operation);

            request.setRequestContent(p_Request);

            WsdlSubmitContext wsdlSubmitContext = new WsdlSubmitContext(request);

            WsdlSubmit submit = request.submit(wsdlSubmitContext, false);

            Response response = submit.getResponse();
            result = response.getContentAsString();
        } catch (XmlException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (SoapUIException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (Request.SubmitException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }


        return result;

    }

}
